/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file SectionOffsetReplacer.hh
 *
 * Declaration of SectionOffsetReplacer class.
 *
 * @author Mikael Lepist 2003 (tmlepist-no.spam-cs.tut.fi)
 *
 * @note rating: yellow
 */

#ifndef TTA_SECTION_OFFSET_REPLACER_HH
#define TTA_SECTION_OFFSET_REPLACER_HH

#include "ValueReplacer.hh"
#include "Exception.hh"

namespace TPEF {

/**
 * Replaces object reference with section offset. Section offset key
 * must be stored in reference manager for that object whose section
 * offset is written.
 */
class SectionOffsetReplacer : public ValueReplacer {
public:
    SectionOffsetReplacer(const SafePointable* obj);
    virtual ~SectionOffsetReplacer();

protected:
    virtual bool tryToReplace()
        throw (UnreachableStream, WritePastEOF);

    virtual ValueReplacer* clone();

private:
    SectionOffsetReplacer(const SectionOffsetReplacer& replacer);
    SectionOffsetReplacer operator=(const SectionOffsetReplacer&);
};
}

#endif
